package com.kalyshev.yan.cabinet.service;

import com.kalyshev.yan.cabinet.model.Cabinet;
import com.kalyshev.yan.cabinet.repository.CabinetNotFoundException;
import com.kalyshev.yan.cabinet.repository.CabinetRepository;
import com.kalyshev.yan.computer.model.Computer;
import com.kalyshev.yan.computer.service.ComputerService;
import com.kalyshev.yan.util.validation.ValidatorUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Optional;

@Service
public class CabinetService {
    private final CabinetRepository cabinetRepository;
    private final ComputerService computerService;
    private final ValidatorUtil validatorUtil;
    public CabinetService(CabinetRepository cabinetRepository,
                          ComputerService computerService,
                          ValidatorUtil validatorUtil) {
        this.cabinetRepository = cabinetRepository;
        this.computerService = computerService;
        this.validatorUtil = validatorUtil;
    }
    @Transactional
    public Cabinet addCabinet(String number) {
        if (!StringUtils.hasText(number)) {
            throw new IllegalArgumentException("Cabinet number is null or empty");
        }
        final Cabinet cabinet = new Cabinet(number);
        validatorUtil.validate(cabinet);
        return cabinetRepository.save(cabinet);
    }
    @Transactional(readOnly = true)
    public Cabinet findCabinet(Long id) {
        final Optional<Cabinet> cabinet = cabinetRepository.findById(id);
        return cabinet.orElseThrow(() -> new CabinetNotFoundException(id));
    }
    @Transactional(readOnly = true)
    public List<Cabinet> findFilteredCabinets(Long id, String number) {
        return cabinetRepository.findFilteredCabinets(id, number);
    }
    @Transactional(readOnly = true)
    public List<Cabinet> findAllCabinets() {
        return cabinetRepository.findAll();
    }
    @Transactional
    public Cabinet updateCabinet(Long id, String number) {
        if (!StringUtils.hasText(number)) {
            throw new IllegalArgumentException("Cabinet number is null or empty");
        }
        final Cabinet currentCabinet = findCabinet(id);
        currentCabinet.setNumber(number);
        validatorUtil.validate(currentCabinet);
        return cabinetRepository.save(currentCabinet);
    }
    @Transactional
    public Cabinet deleteCabinet(Long id) {
        final Cabinet currentCabinet = findCabinet(id);
        computerService.deleteRelationsWithCabinets(currentCabinet.getComputers());
        cabinetRepository.delete(currentCabinet);
        return currentCabinet;
    }
    @Transactional
    public void deleteAllCabinets() {
        cabinetRepository.deleteAll();
    }
    @Transactional
    public List<Computer> listComputersFromCabinet(Long id) {
        if ((Object)id == null) {
            throw new IllegalArgumentException("Cabinet id is null or empty");
        }
        return findCabinet(id).getComputers();
    }
    @Transactional
    public void addComputerToCabinet(Long computerId, Long cabinetId) {
        if ((Object)computerId == null) {
            throw new IllegalArgumentException("Computer id is null or empty");
        }
        if ((Object)cabinetId == null) {
            throw new IllegalArgumentException("Cabinet id is null or empty");
        }
        final Computer computer = computerService.findComputer(computerId);
        final Cabinet cabinet = findCabinet(cabinetId);
        cabinet.addComputer(computer);
    }
    @Transactional
    public void deleteComputerFromCabinet(Long computerId, Long cabinetId) {
        if ((Object) computerId == null) {
            throw new IllegalArgumentException("Computer id is null or empty");
        }
        if ((Object) cabinetId == null) {
            throw new IllegalArgumentException("Cabinet id is null or empty");
        }
        final Computer computer = computerService.findComputer(computerId);
        final Cabinet cabinet = findCabinet(cabinetId);
        cabinet.removeComputer(computer);
    }
}